@@ -1791,8 +1791,8 @@ gfxFontUtils::MakeEOTHeader(const PRUint8 *aFontData, PRUint32 aFontDataLength,
             break;
     }
 
-    if (needNames != 0) 
-    {
+    // the Version name is allowed to be null
+    if ((needNames & ~(1 << EOTFixedHeader::EOT_VERSION_NAME_INDEX)) != 0) {
         return NS_ERROR_FAILURE;
     }
 
