@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Gary Kwong
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,14 +35,13 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-387951-03.js';
+var gTestfile = 'regress-452724-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 387951;
-var summary = 'Do not assert: cg->stackDepth >= 0';
+var BUGNUMBER = 452724;
+var summary = 'Do not assert with JIT: (rmask(rr) & FpRegs) != 0';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
-
 //-----------------------------------------------------------------------------
 test();
 //-----------------------------------------------------------------------------
@@ -52,8 +51,12 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  switch(delete[null?0:{}]){default:}
+
+  jit(true);
+
+  (function() { for (var j=0;j<5;++j) { (0/0) in this; } })()
+
+  jit(false);
 
   reportCompare(expect, actual, summary);
 
