@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jason Orendorff
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,10 +35,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-469625-01.js';
+var gTestfile = 'regress-352873-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 469625;
-var summary = 'decompile [a, b, [c0, c1]] = [x, x, x];';
+var BUGNUMBER = 352873;
+var summary = 'decompilation of nested |try...catch| with |with|';
 var actual = '';
 var expect = '';
 
@@ -52,15 +52,17 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  function f(x) {
-    var [a, b, [c0, c1]] = [x, x, x];
-  }
 
-  expect = 'function f(x) { var [a, b, [c0, c1]] = [x, x, x]; }';
+  var f;
+
+  f = function() { try { with({}) { try { } finally { return; }  }  } finally { } }
+  expect = 'function() { try { with({}) { try { } finally { return; }  }  } finally { } }';
   actual = f + '';
-
   compareSource(expect, actual, summary);
 
+  expect = actual = 'No Crash';
+  f();
+  reportCompare(expect, actual, summary);
+
   exitFunc ('test');
 }
