@@ -56,19 +56,29 @@
 # define W_OK 02
 # define R_OK 04
 # define access _access
+#ifndef WINCE
 # define putenv _putenv
+#endif
 # define snprintf _snprintf
 # define fchmod(a,b)
-# define mkdir(path, perms) _mkdir(path)
 
 # define NS_T(str) L ## str
 # define NS_tsnprintf _snwprintf
 # define NS_tstrrchr wcsrchr
+# define NS_taccess _waccess
 # define NS_tchdir _wchdir
+# define NS_tchmod _wchmod
+# define NS_tmkdir(path, perms) _wmkdir(path)
 # define NS_tremove _wremove
-# define NS_topen _wopen
 # define NS_tfopen _wfopen
 # define NS_tatoi _wtoi64
+#ifndef WINCE
+# define stat _stat
+#endif
+# define NS_tstat _wstat
+# define BACKUP_EXT L".moz-backup"
+# define CALLBACK_BACKUP_EXT L".moz-callback"
+# define LOG_S "%S"
 #else
 # include <sys/wait.h>
 # include <unistd.h>
@@ -76,11 +86,16 @@
 # define NS_T(str) str
 # define NS_tsnprintf snprintf
 # define NS_tstrrchr strrchr
+# define NS_taccess access
 # define NS_tchdir chdir
+# define NS_tchmod chmod
+# define NS_tmkdir mkdir
 # define NS_tremove remove
-# define NS_topen open
 # define NS_tfopen fopen
 # define NS_tatoi atoi
+# define NS_tstat stat
+# define BACKUP_EXT ".moz-backup"
+# define LOG_S "%s"
 #endif
 
 #include "bspatch.h"
@@ -100,6 +115,16 @@
 #include <limits.h>
 #include <errno.h>
 
+#ifdef WINCE
+#include "updater_wince.h"
+#endif
+
+// Amount of the progress bar to use in each of the 3 update stages,
+// should total 100.0.
+#define PROGRESS_PREPARE_SIZE 20.0f
+#define PROGRESS_EXECUTE_SIZE 75.0f
+#define PROGRESS_FINISH_SIZE   5.0f
+
 #if defined(XP_MACOSX)
 // This function is defined in launchchild_osx.mm
 void LaunchChild(int argc, char **argv);
@@ -120,7 +145,7 @@ void LaunchChild(int argc, char **argv);
 #ifndef MAXPATHLEN
 # ifdef PATH_MAX
 #  define MAXPATHLEN PATH_MAX
-# elif defined(_MAX_PATH)
+# elif defined(MAX_PATH)
 #  define MAXPATHLEN MAX_PATH
 # elif defined(_MAX_PATH)
 #  define MAXPATHLEN _MAX_PATH
@@ -137,6 +162,28 @@ void LaunchChild(int argc, char **argv);
 #define USE_EXECV
 #endif
 
+#ifdef XP_WIN
+#ifdef WINCE
+#define EXIT_WHEN_ELEVATED(path, handle, retCode)
+#else
+// Closes the handle if valid and if the updater is elevated returns with the
+// return code specified. This prevents multiple launches of the callback
+// application by preventing the elevated process from launching the callback.
+#define EXIT_WHEN_ELEVATED(path, handle, retCode) \
+  { \
+      if (handle != INVALID_HANDLE_VALUE) { \
+        CloseHandle(handle); \
+      } \
+      if (_waccess(path, F_OK) == 0 && NS_tremove(path) != 0) { \
+        return retCode; \
+      } \
+  }
+#endif
+#endif
+
+char *BigBuffer = NULL;
+int BigBufferSize = 262144;
+
 //-----------------------------------------------------------------------------
 
 // This variable lives in libbz2.  It's declared in bzlib_private.h, so we just
@@ -164,33 +211,37 @@ crc32(const unsigned char *buf, unsigned int len)
 
 //-----------------------------------------------------------------------------
 
-// A simple stack based container for a file descriptor (int) that closes the
+// A simple stack based container for a FILE struct that closes the
 // file descriptor from its destructor.
-class AutoFD
+class AutoFile
 {
 public:
-  AutoFD(int fd = -1)
-    : mFD(fd) {
+  AutoFile(FILE* file = NULL)
+    : mFile(file) {
   }
 
-  ~AutoFD() {
-    if (mFD != -1)
-      close(mFD);
+  ~AutoFile() {
+    if (mFile != NULL)
+      fclose(mFile);
   }
 
-  AutoFD &operator=(int fd) {
-    if (mFD != -1)
-      close(mFD);
-    mFD = fd;
+  AutoFile &operator=(FILE* file) {
+    if (mFile != 0)
+      fclose(mFile);
+    mFile = file;
     return *this;
   }
 
-  operator int() {
-    return mFD;
+  operator FILE*() {
+    return mFile;
+  }
+  
+  FILE* get() {
+    return mFile;
   }
 
 private:
-  int mFD;
+  FILE* mFile;
 };
 
 //-----------------------------------------------------------------------------
@@ -208,7 +259,11 @@ public:
     mThreadFunc = func;
     mThreadParam = param;
 
-    unsigned threadID;
+#ifdef WINCE
+    DWORD threadID;
+#else
+    unsigned int threadID;
+#endif
     mThread = (HANDLE) _beginthreadex(NULL, 0, ThreadMain, this, 0, &threadID);
     
     return mThread ? 0 : -1;
@@ -289,8 +344,15 @@ private:
 
 static NS_tchar* gSourcePath;
 static ArchiveReader gArchiveReader;
+
 #ifdef XP_WIN
 static bool gSucceeded = FALSE;
+WIN32_FIND_DATAW gFFData;
+#ifdef WINCE
+// Since WinCE doesn't have a current working directory store the current
+// working directory specified in the command line arguments 
+static NS_tchar* gDestPath;
+#endif
 #endif
 
 static const char kWhitespace[] = " \t";
@@ -382,10 +444,42 @@ mstrtok(const char *delims, char **str)
   return ret;
 }
 
-static void ensure_write_permissions(const char *path)
+#ifdef XP_WIN
+// Returns a wchar path. The path returned will be absolute on Windows CE /
+// Windows Mobile and relative on all other versions of Windows.
+static NS_tchar*
+get_wide_path(const char *path)
+{
+  WCHAR *s = (WCHAR*) malloc(MAXPATHLEN * sizeof(WCHAR));
+  if (!s)
+    return NULL;
+
+  WCHAR *c = s;
+  WCHAR wpath[MAXPATHLEN];
+  MultiByteToWideChar(CP_ACP, 0, path, -1, wpath, MAXPATHLEN);
+#ifndef WINCE
+  wcscpy(c, wpath);
+  c += wcslen(wpath);
+#else
+  wcscpy(c, gDestPath);
+  c += wcslen(gDestPath);
+  wcscat(c, wpath);
+  c += wcslen(wpath);
+#endif
+  if (wcscmp(wpath, gFFData.cFileName) == 0) {
+    wcscat(c, CALLBACK_BACKUP_EXT);
+    c += wcslen(CALLBACK_BACKUP_EXT);
+  }
+  *c = NS_T('\0');
+  c++;
+  return s;
+}
+#endif
+
+static void ensure_write_permissions(const NS_tchar *path)
 {
 #ifdef XP_WIN
-  (void) chmod(path, _S_IREAD | _S_IWRITE);
+  (void) _wchmod(path, _S_IREAD | _S_IWRITE);
 #else
   struct stat fs;
   if (!stat(path, &fs) && !(fs.st_mode & S_IWUSR)) {
@@ -394,46 +488,60 @@ static void ensure_write_permissions(const char *path)
 #endif
 }
 
-static int ensure_remove(const char *path)
+static int ensure_remove(const NS_tchar *path)
 {
   ensure_write_permissions(path);
-  int rv = remove(path);
+  int rv = NS_tremove(path);
   if (rv)
-    LOG(("remove failed: %d,%d (%s)\n", rv, errno, path));
+    LOG(("remove failed: %d,%d (" LOG_S ")\n", rv, errno, path));
   return rv;
 }
 
-static int ensure_open(const char *path, int flags, int options)
+static FILE* ensure_open(const NS_tchar *path, const NS_tchar* flags, unsigned int options)
 {
   ensure_write_permissions(path);
-  return open(path, flags, options);
+  FILE* f = NS_tfopen(path, flags);
+  if (NS_tchmod(path, options) != 0) {
+    if (f != NULL) {
+      fclose(f);
+    }
+    return NULL;
+  }
+  struct stat ss;
+  if (NS_tstat(path, &ss) != 0 || ss.st_mode != options) {
+    if (f != NULL) {
+      fclose(f);
+    }
+    return NULL;
+  }
+  return f;
 }
 
 // Ensure that the directory containing this file exists.
-static int ensure_parent_dir(const char *path)
+static int ensure_parent_dir(const NS_tchar *path)
 {
   int rv = OK;
 
-  char *slash = (char *) strrchr(path, '/');
-  if (slash)
-  {
-    *slash = '\0';
+  NS_tchar *slash = (NS_tchar *) NS_tstrrchr(path, NS_T('/'));
+  if (slash) {
+    *slash = NS_T('\0');
     rv = ensure_parent_dir(path);
     if (rv == OK) {
-      rv = mkdir(path, 0755);
-      // If the directory already exists, then ignore the error.
+      rv = NS_tmkdir(path, 0755);
+      // If the directory already exists, then ignore the error. On WinCE rv
+      // will equal 0 if the directory already exists.
       if (rv < 0 && errno != EEXIST) {
         rv = WRITE_ERROR;
       } else {
         rv = OK;
       }
     }
-    *slash = '/';
+    *slash = NS_T('/');
   }
   return rv;
 }
 
-static int copy_file(const char *spath, const char *dpath)
+static int copy_file(const NS_tchar *spath, const NS_tchar *dpath)
 {
   int rv = ensure_parent_dir(dpath);
   if (rv)
@@ -441,24 +549,23 @@ static int copy_file(const char *spath, const char *dpath)
 
   struct stat ss;
 
-  AutoFD sfd = open(spath, O_RDONLY | _O_BINARY);
-  if (sfd < 0 || fstat(sfd, &ss)) {
-    LOG(("copy_file: failed to open or stat: %d,%s,%d\n", (int) sfd, spath, errno));
+  AutoFile sfile = NS_tfopen(spath, NS_T("rb"));
+  if (sfile == NULL || fstat(fileno(sfile), &ss)) {
+    LOG(("copy_file: failed to open or stat: %p," LOG_S ",%d\n", sfile.get(), spath, errno));
     return READ_ERROR;
   }
 
-  AutoFD dfd = ensure_open(dpath, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, ss.st_mode);
-  if (dfd < 0) {
-    LOG(("copy_file: failed to open: %s,%d\n", dpath, errno));
+  AutoFile dfile = ensure_open(dpath, NS_T("wb+"), ss.st_mode); 
+  if (dfile == NULL) {
+    LOG(("copy_file: failed to open: " LOG_S ",%d\n", dpath, errno));
     return WRITE_ERROR;
   }
 
-  char buf[BUFSIZ];
   int sc;
-  while ((sc = read(sfd, buf, sizeof(buf))) > 0) {
+  while ((sc = fread(BigBuffer, 1, BigBufferSize, sfile)) > 0) {
     int dc;
-    char *bp = buf;
-    while ((dc = write(dfd, bp, (unsigned int) sc)) > 0) {
+    char *bp = BigBuffer;
+    while ((dc = fwrite(bp, 1, (unsigned int) sc, dfile)) > 0) {
       if ((sc -= dc) == 0)
         break;
       bp += dc;
@@ -478,13 +585,11 @@ static int copy_file(const char *spath, const char *dpath)
 
 //-----------------------------------------------------------------------------
 
-#define BACKUP_EXT ".moz-backup"
-
 // Create a backup copy of the specified file alongside it.
-static int backup_create(const char *path)
+static int backup_create(const NS_tchar *path)
 {
-  char backup[MAXPATHLEN];
-  snprintf(backup, sizeof(backup), "%s" BACKUP_EXT, path);
+  NS_tchar backup[MAXPATHLEN];
+  NS_tsnprintf(backup, sizeof(backup), NS_T("%s" BACKUP_EXT), path);
 
   return copy_file(path, backup);
 }
@@ -492,27 +597,30 @@ static int backup_create(const char *path)
 // Copy the backup copy of the specified file back overtop
 // the specified file.
 // XXX should be a file move instead
-static int backup_restore(const char *path)
+static int backup_restore(const NS_tchar *path)
 {
-  char backup[MAXPATHLEN];
-  snprintf(backup, sizeof(backup), "%s" BACKUP_EXT, path);
+  NS_tchar backup[MAXPATHLEN];
+  NS_tsnprintf(backup, sizeof(backup), NS_T("%s" BACKUP_EXT), path);
 
   int rv = copy_file(backup, path);
-  if (rv)
+  if (rv) {
+    ensure_remove(backup);
     return rv;
+  }
 
   rv = ensure_remove(backup);
-  if (rv)
+  if (rv) {
     return WRITE_ERROR;
+  }
 
   return OK;
 }
 
 // Discard the backup copy of the specified file.
-static int backup_discard(const char *path)
+static int backup_discard(const NS_tchar *path)
 {
-  char backup[MAXPATHLEN];
-  snprintf(backup, sizeof(backup), "%s" BACKUP_EXT, path);
+  NS_tchar backup[MAXPATHLEN];
+  NS_tsnprintf(backup, sizeof(backup), NS_T("%s" BACKUP_EXT), path);
 
   int rv = ensure_remove(backup);
   if (rv)
@@ -522,7 +630,7 @@ static int backup_discard(const char *path)
 }
 
 // Helper function for post-processing a temporary backup.
-static void backup_finish(const char *path, int status)
+static void backup_finish(const NS_tchar *path, int status)
 {
   if (status == OK)
     backup_discard(path);
@@ -539,7 +647,7 @@ static const int ACTION_DESCRIPTION_BUFSIZE = 256;
 class Action
 {
 public:
-  Action() : mNext(NULL) { }
+  Action() : mProgressCost(1), mNext(NULL) { }
   virtual ~Action() { }
 
   virtual int Parse(char *line) = 0;
@@ -557,6 +665,7 @@ public:
   // all actions were successfully executed.  Otherwise, some action failed.
   virtual void Finish(int status) = 0;
 
+  int mProgressCost;
 private:
   Action* mNext;
 
@@ -566,7 +675,7 @@ private:
 class RemoveFile : public Action
 {
 public:
-  RemoveFile() : mFile(NULL), mSkip(0) { }
+  RemoveFile() : mFile(NULL), mDestFile(NULL), mSkip(0) { }
 
   int Parse(char *line);
   int Prepare();
@@ -575,6 +684,7 @@ public:
 
 private:
   const char* mFile;
+  const NS_tchar* mDestFile;
   int mSkip;
 };
 
@@ -587,6 +697,14 @@ RemoveFile::Parse(char *line)
   if (!mFile)
     return PARSE_ERROR;
 
+#ifdef XP_WIN
+  mDestFile = get_wide_path(mFile);
+  if (!mDestFile)
+    return PARSE_ERROR;
+#else
+  mDestFile = mFile;
+#endif
+
   return OK;
 }
 
@@ -596,26 +714,29 @@ RemoveFile::Prepare()
   LOG(("PREPARE REMOVE %s\n", mFile));
 
   // We expect the file to exist if we are to remove it.
-  int rv = access(mFile, F_OK);
+  int rv = NS_taccess(mDestFile, F_OK);
   if (rv) {
-    LOG(("file cannot be removed because it does not exist; skipping\n"));
+    LOG(("file does not exist; skipping\n"));
     mSkip = 1;
+    mProgressCost = 0;
     return OK;
   }
 
-  char *slash = (char *) strrchr(mFile, '/');
+#ifndef WINCE
+  NS_tchar *slash = (NS_tchar *) NS_tstrrchr(mDestFile, NS_T('/'));
   if (slash) {
-    *slash = '\0';
-    rv = access(mFile, W_OK);
-    *slash = '/';
+    *slash = NS_T('\0');
+    rv = NS_taccess(mDestFile, W_OK);
+    *slash = NS_T('/');
   } else {
-    rv = access(".", W_OK);
+    rv = NS_taccess(NS_T("."), W_OK);
   }
 
   if (rv) {
     LOG(("access failed: %d\n", errno));
     return WRITE_ERROR;
   }
+#endif
 
   return OK;
 }
@@ -623,15 +744,15 @@ RemoveFile::Prepare()
 int
 RemoveFile::Execute()
 {
-  LOG(("EXECUTE REMOVE %s\n", mFile));
-
   if (mSkip)
     return OK;
 
+  LOG(("EXECUTE REMOVE %s\n", mFile));
+
   // We expect the file to exist if we are to remove it.  We check here as well
   // as in PREPARE since we might have been asked to remove the same file more
   // than once: bug 311099.
-  int rv = access(mFile, F_OK);
+  int rv = NS_taccess(mDestFile, F_OK);
   if (rv) {
     LOG(("file cannot be removed because it does not exist; skipping\n"));
     mSkip = 1;
@@ -641,13 +762,13 @@ RemoveFile::Execute()
   // save a complete copy of the old file, and then remove the
   // old file.  we'll clean up the copy in Finish.
 
-  rv = backup_create(mFile);
+  rv = backup_create(mDestFile);
   if (rv) {
     LOG(("backup_create failed: %d\n", rv));
     return rv;
   }
 
-  rv = ensure_remove(mFile);
+  rv = ensure_remove(mDestFile);
   if (rv)
     return WRITE_ERROR;
 
@@ -657,18 +778,18 @@ RemoveFile::Execute()
 void
 RemoveFile::Finish(int status)
 {
-  LOG(("FINISH REMOVE %s\n", mFile));
-
   if (mSkip)
     return;
 
-  backup_finish(mFile, status);
+  LOG(("FINISH REMOVE %s\n", mFile));
+
+  backup_finish(mDestFile, status);
 }
 
 class AddFile : public Action
 {
 public:
-  AddFile() : mFile(NULL) { }
+  AddFile() : mFile(NULL), mDestFile(NULL) { }
 
   virtual int Parse(char *line);
   virtual int Prepare(); // check that the source file exists
@@ -677,6 +798,7 @@ public:
 
 private:
   const char *mFile;
+  const NS_tchar *mDestFile;
 };
 
 int
@@ -688,6 +810,14 @@ AddFile::Parse(char *line)
   if (!mFile)
     return PARSE_ERROR;
 
+#ifdef XP_WIN
+  mDestFile = get_wide_path(mFile);
+  if (!mDestFile)
+    return PARSE_ERROR;
+#else
+  mDestFile = mFile;
+#endif
+
   return OK;
 }
 
@@ -707,24 +837,22 @@ AddFile::Execute()
   int rv;
 
   // First make sure that we can actually get rid of any existing file.
-  if (access(mFile, F_OK) == 0)
-  {
-    rv = backup_create(mFile);
+  rv = NS_taccess(mDestFile, F_OK);
+  if (rv == 0) {
+    rv = backup_create(mDestFile);
     if (rv)
       return rv;
 
-    rv = ensure_remove(mFile);
+    rv = ensure_remove(mDestFile);
     if (rv)
       return WRITE_ERROR;
-  }
-  else
-  {
-    rv = ensure_parent_dir(mFile);
+  } else {
+    rv = ensure_parent_dir(mDestFile);
     if (rv)
       return rv;
   }
-    
-  return gArchiveReader.ExtractFile(mFile, mFile);
+
+  return gArchiveReader.ExtractFile(mFile, mDestFile);
 }
 
 void
@@ -732,13 +860,13 @@ AddFile::Finish(int status)
 {
   LOG(("FINISH ADD %s\n", mFile));
 
-  backup_finish(mFile, status);
+  backup_finish(mDestFile, status);
 }
 
 class PatchFile : public Action
 {
 public:
-  PatchFile() : mPatchIndex(-1), pfd(-1), buf(NULL) { }
+  PatchFile() : mPatchIndex(-1), buf(NULL) { }
   virtual ~PatchFile();
 
   virtual int Parse(char *line);
@@ -747,40 +875,36 @@ public:
   virtual void Finish(int status);
 
 private:
-  int LoadSourceFile(int ofd);
+  int LoadSourceFile(FILE* ofile);
 
   static int sPatchIndex;
 
   const char *mPatchFile;
   const char *mFile;
+  const NS_tchar *mDestFile;
   int mPatchIndex;
   MBSPatchHeader header;
-  int pfd;
   unsigned char *buf;
+  NS_tchar spath[MAXPATHLEN];
 };
 
 int PatchFile::sPatchIndex = 0;
 
 PatchFile::~PatchFile()
 {
-  if (pfd >= 0)
-    close(pfd);
-
   // delete the temporary patch file
-  NS_tchar spath[MAXPATHLEN];
-  NS_tsnprintf(spath, MAXPATHLEN, NS_T("%s/%d.patch"),
-               gSourcePath, mPatchIndex);
+  if (spath[0])
+    NS_tremove(spath);
 
-  NS_tremove(spath);
-
-  free(buf);
+  if (buf)
+    free(buf);
 }
 
 int
-PatchFile::LoadSourceFile(int ofd)
+PatchFile::LoadSourceFile(FILE* ofile)
 {
   struct stat os;
-  int rv = fstat(ofd, &os);
+  int rv = fstat(fileno(ofile), &os);
   if (rv)
     return READ_ERROR;
 
@@ -794,7 +918,7 @@ PatchFile::LoadSourceFile(int ofd)
   int r = header.slen;
   unsigned char *rb = buf;
   while (r) {
-    int c = read(ofd, rb, mmin(BUFSIZ,r));
+    int c = fread(rb, 1, r, ofile);
     if (c < 0)
       return READ_ERROR;
 
@@ -835,6 +959,14 @@ PatchFile::Parse(char *line)
   if (!mFile)
     return PARSE_ERROR;
 
+#ifdef XP_WIN
+  mDestFile = get_wide_path(mFile);
+  if (!mDestFile)
+    return PARSE_ERROR;
+#else
+  mDestFile = mFile;
+#endif
+
   return OK;
 }
 
@@ -846,7 +978,6 @@ PatchFile::Prepare()
   // extract the patch to a temporary file
   mPatchIndex = sPatchIndex++;
 
-  NS_tchar spath[MAXPATHLEN];
   NS_tsnprintf(spath, MAXPATHLEN, NS_T("%s/%d.patch"),
                gSourcePath, mPatchIndex);
 
@@ -858,28 +989,6 @@ PatchFile::Prepare()
 
   int rv = gArchiveReader.ExtractFileToStream(mPatchFile, fp);
   fclose(fp);
-  if (rv)
-    return rv;
-
-  // XXXdarin from here down should be moved into the Execute command.
-  //          no need to open all of the patch files and read all of 
-  //          the source files before applying any patches.
-
-  pfd = NS_topen(spath, O_RDONLY | _O_BINARY);
-  if (pfd < 0)
-    return READ_ERROR;
-
-  rv = MBS_ReadHeader(pfd, &header);
-  if (rv)
-    return rv;
-
-  AutoFD ofd = open(mFile, O_RDONLY | _O_BINARY);
-  if (ofd < 0)
-    return READ_ERROR;
-
-  rv = LoadSourceFile(ofd);
-  if (rv)
-    LOG(("LoadSourceFile failed\n"));
   return rv;
 }
 
@@ -888,25 +997,52 @@ PatchFile::Execute()
 {
   LOG(("EXECUTE PATCH %s\n", mFile));
 
-  // Create backup copy of the destination file before proceeding.
-
-  struct stat ss;
-  if (stat(mFile, &ss))
+  AutoFile pfile = NS_tfopen(spath, NS_T("rb"));
+  if (pfile == NULL)
     return READ_ERROR;
 
-  int rv = backup_create(mFile);
+  int rv = MBS_ReadHeader(pfile, &header);
   if (rv)
     return rv;
 
-  rv = ensure_remove(mFile);
+  FILE *origfile = NS_tfopen(mDestFile, NS_T("rb"));
+  if (!origfile)
+    return READ_ERROR;
+
+  rv = LoadSourceFile(origfile);
+  fclose(origfile);
+  if (rv) {
+    LOG(("LoadSourceFile failed\n"));
+    return rv;
+  }
+
+  // Create backup copy of the destination file before proceeding.
+
+  struct stat ss;
+  if (NS_tstat(mDestFile, &ss))
+    return READ_ERROR;
+
+  rv = backup_create(mDestFile);
+  if (rv)
+    return rv;
+
+  rv = ensure_remove(mDestFile);
   if (rv)
     return WRITE_ERROR;
 
-  AutoFD ofd = ensure_open(mFile, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, ss.st_mode);
-  if (ofd < 0)
+  AutoFile ofile = ensure_open(mDestFile, NS_T("wb+"), ss.st_mode);
+  if (ofile == NULL)
     return WRITE_ERROR;
 
-  return MBS_ApplyPatch(&header, pfd, buf, ofd);
+  rv = MBS_ApplyPatch(&header, pfile, buf, ofile);
+
+  // Go ahead and do a bit of cleanup now to minimize runtime overhead.
+  NS_tremove(spath);
+  spath[0] = '\0';
+  free(buf);
+  buf = NULL;
+
+  return rv;
 }
 
 void
@@ -914,7 +1050,7 @@ PatchFile::Finish(int status)
 {
   LOG(("FINISH PATCH %s\n", mFile));
 
-  backup_finish(mFile, status);
+  backup_finish(mDestFile, status);
 }
 
 class AddIfFile : public AddFile
@@ -1044,15 +1180,8 @@ PatchIfFile::Finish(int status)
 #ifdef XP_WIN
 #include "nsWindowsRestart.cpp"
 
-static void
-copyASCIItoWCHAR(WCHAR *dest, const char *src)
-{
-  while (*src) {
-    *dest = *src;
-    ++src; ++dest;
-  }
-}
-
+#ifndef WINCE // until we have a replacement for GetPrivateProfileStringW, 
+              // it doesn't make sense to use this function
 static void
 LaunchWinPostProcess(const WCHAR *appExe)
 {
@@ -1069,7 +1198,8 @@ LaunchWinPostProcess(const WCHAR *appExe)
 
   WCHAR exefile[MAXPATHLEN];
   WCHAR exearg[MAXPATHLEN];
-
+  WCHAR exeasync[10];
+  PRBool async = PR_TRUE;
   if (!GetPrivateProfileStringW(L"PostUpdateWin", L"ExeRelPath", NULL, exefile,
                                 MAXPATHLEN, inifile))
     return;
@@ -1078,6 +1208,10 @@ LaunchWinPostProcess(const WCHAR *appExe)
                                 MAXPATHLEN, inifile))
     return;
 
+  if (!GetPrivateProfileStringW(L"PostUpdateWin", L"ExeAsync", L"TRUE", exeasync,
+                                sizeof(exeasync)/sizeof(exeasync[0]), inifile))
+    return;
+
   WCHAR exefullpath[MAXPATHLEN];
   wcscpy(exefullpath, appExe);
 
@@ -1093,38 +1227,119 @@ LaunchWinPostProcess(const WCHAR *appExe)
   WCHAR slogFile[MAXPATHLEN];
   _snwprintf(slogFile, MAXPATHLEN, L"%s/update.log", gSourcePath);
 
+  WCHAR dummyArg[13];
+  wcscpy(dummyArg, L"argv0ignored ");
+
+  int len = wcslen(exearg) + wcslen(dummyArg);
+  WCHAR *cmdline = (WCHAR *) malloc((len + 1) * sizeof(WCHAR));
+  if (!cmdline)
+    return;
+
+  wcscpy(cmdline, dummyArg);
+  wcscat(cmdline, exearg);
+
+  if (!_wcsnicmp(exeasync, L"false", 6) || !_wcsnicmp(exeasync, L"0", 2))
+    async = PR_FALSE;
+
   // We want to launch the post update helper app to update the Windows
   // registry even if there is a failure with removing the uninstall.update
   // file or copying the update.log file.
   NS_tremove(dlogFile);
   CopyFile(slogFile, dlogFile, FALSE);
 
-  static int    argc = 2;
-  static WCHAR* argv[3] = {
-    L"argv0ignoredbywinlaunchchild",
-    exearg,
-    L"\0"
-  };
- 
-  WinLaunchChild(exefullpath, argc, argv, 0);
+  STARTUPINFOW si = {sizeof(si), 0};
+  PROCESS_INFORMATION pi = {0};
+
+  BOOL ok = CreateProcessW(exefullpath,
+                           cmdline,
+                           NULL,  // no special security attributes
+                           NULL,  // no special thread attributes
+                           FALSE, // don't inherit filehandles
+                           0,     // No special process creation flags
+                           NULL,  // inherit my environment
+                           NULL,  // use my current directory
+                           &si,
+                           &pi);
+  free(cmdline);
+
+  if (ok) {
+    if (!async)
+      WaitForSingleObject(pi.hProcess, INFINITE);
+
+    CloseHandle(pi.hProcess);
+    CloseHandle(pi.hThread);
+  }
+}
+#endif // WINCE
+#endif
+
+#ifdef WINCE
+static char*
+AllocConvertUTF16toUTF8(const WCHAR *arg)
+{
+  // be generous... UTF16 units can expand up to 3 UTF8 units
+  int len = wcslen(arg);
+  char *s = new char[len * 3 + 1];
+  if (!s) {
+    return NULL;
+  }
+  ConvertUTF16toUTF8 convert(s);
+  convert.write(arg, len);
+  convert.write_terminator();
+  return s;
 }
 #endif
 
 static void
 LaunchCallbackApp(const NS_tchar *workingDir, int argc, NS_tchar **argv)
 {
-  putenv("NO_EM_RESTART=");
-  putenv("MOZ_LAUNCHED_CHILD=1");
+  putenv(const_cast<char*>("NO_EM_RESTART="));
+  putenv(const_cast<char*>("MOZ_LAUNCHED_CHILD=1"));
 
-  // Run from the specified working directory (see bug 312360).
-  NS_tchdir(workingDir);
+#ifndef WINCE
+  // Run from the specified working directory (see bug 312360). This is not
+  // necessary on Windows CE since the application that launches the updater
+  // passes the working directory as an --environ: command line argument.
+  if(NS_tchdir(workingDir) != 0)
+    LOG(("Warning: chdir failed\n"));
+#endif
 
 #if defined(USE_EXECV)
   execv(argv[0], argv);
 #elif defined(XP_MACOSX)
   LaunchChild(argc, argv);
-#elif defined(XP_WIN)
-  WinLaunchChild(argv[0], argc, argv, 0);
+#elif defined(XP_WIN) && !defined(WINCE)
+  WinLaunchChild(argv[0], argc, argv);
+#elif defined(WINCE)
+  // Support the mock environment used on Windows CE.
+  int i;
+  int winceArgc = 0;
+  WCHAR **winceArgv;
+  for (i = 0; i < argc; ++i) {
+    if (wcsncmp(argv[i], L"--environ:", 10) != 0) {
+      winceArgc++;
+    }
+  }
+
+  winceArgv = (WCHAR**) malloc(sizeof(WCHAR*) * (winceArgc + 1));
+  if (!winceArgv) {
+    return;
+  }
+
+  int c = 0;
+  for (i = 0; i < argc; ++i) {
+    if (wcsncmp(argv[i], L"--environ:", 10) == 0) {
+      char* key_val = AllocConvertUTF16toUTF8(wcsdup(argv[i] + 10));
+      putenv(key_val);
+      delete [] key_val;
+    } else {
+      winceArgv[c] = argv[i];
+      c++;
+    }
+  }
+  winceArgv[winceArgc + 1] = NULL;
+  WinLaunchChild(argv[0], winceArgc, winceArgv);
+  free(winceArgv);
 #else
 # warning "Need implementaton of LaunchCallbackApp"
 #endif
@@ -1138,8 +1353,8 @@ WriteStatusFile(int status)
   NS_tchar filename[MAXPATHLEN];
   NS_tsnprintf(filename, MAXPATHLEN, NS_T("%s/update.status"), gSourcePath);
 
-  AutoFD fd = NS_topen(filename, O_WRONLY | O_TRUNC | O_CREAT | _O_BINARY, 0644);
-  if (fd < 0)
+  AutoFile file = NS_tfopen(filename, NS_T("wb+"));
+  if (file == NULL)
     return;
 
   const char *text;
@@ -1151,7 +1366,7 @@ WriteStatusFile(int status)
     snprintf(buf, sizeof(buf), "failed: %d\n", status);
     text = buf;
   }
-  write(fd, text, strlen(text));
+  fwrite(text, strlen(text), 1, file);
 }
 
 static void
@@ -1181,7 +1396,6 @@ UpdateThreadFunc(void *param)
 int NS_main(int argc, NS_tchar **argv)
 {
   InitProgressUI(&argc, &argv);
-
   // The updater command line consists of the directory path containing the
   // updater.mar file to process followed by the PID of the calling process.
   // The updater will wait on the parent process to exit if the PID is non-
@@ -1189,10 +1403,17 @@ int NS_main(int argc, NS_tchar **argv)
   // necessary for the parent process to exit before its executable image may
   // be altered.
 
+#ifndef WINCE
   if (argc < 2) {
     fprintf(stderr, "Usage: updater <dir-path> [parent-pid [working-dir callback args...]]\n");
     return 1;
   }
+#else
+  if (argc < 4) {
+    fprintf(stderr, "Usage: updater <dir-path> parent-pid <working-dir> [callback args...]]\n");
+    return 1;
+  }
+#endif
 
   if (argc > 2 ) {
     int pid = NS_tatoi(argv[2]);
@@ -1218,15 +1439,22 @@ int NS_main(int argc, NS_tchar **argv)
     }
   }
 
-#ifdef XP_WIN
+  // The callback is the last N command line arguments starting from argOffset.
+  // The argument specified by argOffset is the callback executable and the
+  // argument prior to argOffset is the working directory.
+  const int argOffset = 4;
+
+  gSourcePath = argv[1];
+
+#if defined(XP_WIN) && !defined(WINCE)
   // Launch a second instance of the updater with the runas verb on Windows
   // when write access is denied to the installation directory.
   HANDLE updateLockFileHandle;
   NS_tchar elevatedLockFilePath[MAXPATHLEN];
-  if (argc > 4) {
+  if (argc > argOffset) {
     NS_tchar updateLockFilePath[MAXPATHLEN];
     NS_tsnprintf(updateLockFilePath, MAXPATHLEN,
-                 NS_T("%s.update_in_progress.lock"), argv[4]);
+                 NS_T("%s.update_in_progress.lock"), argv[argOffset]);
 
     // The update_in_progress.lock file should only exist during an update. In
     // case it exists attempt to remove it and exit if that fails to prevent
@@ -1278,8 +1506,8 @@ int NS_main(int argc, NS_tchar **argv)
       SHELLEXECUTEINFO sinfo;
       memset(&sinfo, 0, sizeof(SHELLEXECUTEINFO));
       sinfo.cbSize       = sizeof(SHELLEXECUTEINFO);
-      sinfo.fMask        = SEE_MASK_FLAG_DDEWAIT |
-                           SEE_MASK_FLAG_NO_UI |
+      sinfo.fMask        = SEE_MASK_FLAG_NO_UI |
+                           SEE_MASK_FLAG_DDEWAIT |
                            SEE_MASK_NOCLOSEPROCESS;
       sinfo.hwnd         = NULL;
       sinfo.lpFile       = argv[0];
@@ -1293,10 +1521,13 @@ int NS_main(int argc, NS_tchar **argv)
       if (result) {
         WaitForSingleObject(sinfo.hProcess, INFINITE);
         CloseHandle(sinfo.hProcess);
+      } else {
+        WriteStatusFile(ELEVATION_CANCELED);
       }
 
-      if (argc > 4)
-        LaunchCallbackApp(argv[3], argc - 4, argv + 4);
+      if (argc > argOffset) {
+        LaunchCallbackApp(argv[3], argc - argOffset, argv + argOffset);
+      }
 
       CloseHandle(elevatedFileHandle);
       return 0;
@@ -1304,39 +1535,122 @@ int NS_main(int argc, NS_tchar **argv)
   }
 #endif
 
-  gSourcePath = argv[1];
-
   LogInit();
+  LOG(("SOURCE DIRECTORY " LOG_S "\n", gSourcePath));
+
+#ifdef WINCE
+  // This is the working directory to apply the update and is required on WinCE
+  // since it doesn't have the concept of a working directory.
+  gDestPath = argv[3];
+  LOG(("DESTINATION DIRECTORY " LOG_S "\n", gDestPath));
+#endif
+
+#ifdef XP_WIN
+  HANDLE callbackFile = INVALID_HANDLE_VALUE;
+  NS_tchar callbackBackupPath[MAXPATHLEN];
+  if (argc > argOffset) {
+    // FindFirstFileW is used to get the callback's filename for comparison
+    // with the callback's patch since it will return the correct case and the
+    // long name instead of the 8.3 format name.
+    HANDLE hFindFile;
+    hFindFile = FindFirstFileW(argv[argOffset], &gFFData);
+    if (hFindFile == INVALID_HANDLE_VALUE) {
+      LOG(("NS_main: unable to find callback file: " LOG_S "\n", argv[argOffset]));
+      LogFinish();
+      WriteStatusFile(WRITE_ERROR);
+      EXIT_WHEN_ELEVATED(elevatedLockFilePath, updateLockFileHandle, 1);
+      LaunchCallbackApp(argv[3], argc - argOffset, argv + argOffset);
+      return 1;
+    }
+    FindClose(hFindFile);
+
+    // Make a copy of the callback executable.
+    NS_tsnprintf(callbackBackupPath, sizeof(callbackBackupPath),
+                 NS_T("%s" CALLBACK_BACKUP_EXT), argv[argOffset]);
+    NS_tremove(callbackBackupPath);
+    CopyFileW(argv[argOffset], callbackBackupPath, FALSE);
+
+    // By opening a file handle to the callback executable, the OS will prevent
+    // launching the process while it is being updated. 
+    callbackFile = CreateFileW(argv[argOffset],
+#ifdef WINCE
+                               GENERIC_WRITE,
+                               FILE_SHARE_READ | FILE_SHARE_WRITE,
+#else
+                               DELETE | GENERIC_WRITE,
+                               0, // no sharing!
+#endif
+                               NULL, OPEN_EXISTING, 0, NULL);
+    // CreateFileW will fail if the callback executable is already in use. Since
+    // it isn't possible to update write the status file and return.
+    if (callbackFile == INVALID_HANDLE_VALUE) {
+      LOG(("NS_main: file in use - failed to exclusively open executable " \
+           "file: " LOG_S "\n", argv[argOffset]));
+      LogFinish();
+      WriteStatusFile(WRITE_ERROR);
+      NS_tremove(callbackBackupPath);
+      EXIT_WHEN_ELEVATED(elevatedLockFilePath, updateLockFileHandle, 1);
+      LaunchCallbackApp(argv[3], argc - argOffset, argv + argOffset);
+      return 1;
+    }
+  }
+#endif
+
+  BigBuffer = (char *)malloc(BigBufferSize);
+  if (!BigBuffer) {
+    LOG(("NS_main: failed to allocate default buffer of %i. Trying 1K " \
+         "buffer\n", BigBufferSize));
+
+    // Try again with a smaller size.
+    BigBufferSize = 1024;
+    BigBuffer = (char *)malloc(BigBufferSize);
+    if (!BigBuffer) {
+      LOG(("NS_main: failed to allocate 1K buffer - exiting\n"));
+      LogFinish();
+      WriteStatusFile(MEM_ERROR);
+#ifdef XP_WIN
+      CloseHandle(callbackFile);
+      NS_tremove(callbackBackupPath);
+      EXIT_WHEN_ELEVATED(elevatedLockFilePath, updateLockFileHandle, 1);
+#endif
+      LaunchCallbackApp(argv[3], argc - argOffset, argv + argOffset);
+      return 1;
+    }
+  }
 
   // Run update process on a background thread.  ShowProgressUI may return
   // before QuitProgressUI has been called, so wait for UpdateThreadFunc to
   // terminate.
   Thread t;
-  if (t.Run(UpdateThreadFunc, NULL) == 0)
+  if (t.Run(UpdateThreadFunc, NULL) == 0) {
     ShowProgressUI();
+  }
   t.Join();
 
-  LogFinish();
-
 #ifdef XP_WIN
-  if (gSucceeded && argc > 4)
-    LaunchWinPostProcess(argv[4]);
-
-  if (argc > 4) {
-    CloseHandle(updateLockFileHandle);
-    // If elevated return early and let the process that launched this process
-    // launch the callback application.
-    if (!_waccess(elevatedLockFilePath, F_OK) &&
-        NS_tremove(elevatedLockFilePath) != 0)
-      return 0;
+  if (argc > argOffset) {
+    CloseHandle(callbackFile);
+    // CopyFile will preserve the case of the destination file if it already
+    // exists.
+    if (CopyFileW(callbackBackupPath, argv[argOffset], FALSE) != 0) {
+      NS_tremove(callbackBackupPath);
+    }
   }
 #endif
 
-  // The callback to execute is given as the last N arguments of our command
-  // line.  The first of those arguments specifies the working directory for
-  // the callback.
-  if (argc > 4)
-    LaunchCallbackApp(argv[3], argc - 4, argv + 4);
+  LogFinish();
+  free(BigBuffer);
+  BigBuffer = NULL;
+
+  if (argc > argOffset) {
+#if defined(XP_WIN) && !defined(WINCE)
+    if (gSucceeded) {
+      LaunchWinPostProcess(argv[argOffset]);
+    }
+    EXIT_WHEN_ELEVATED(elevatedLockFilePath, updateLockFileHandle, 0);
+#endif
+    LaunchCallbackApp(argv[3], argc - argOffset, argv + argOffset);
+  }
 
   return 0;
 }
@@ -1392,36 +1706,44 @@ ActionList::Prepare()
   }
 
   Action *a = mFirst;
+  int i = 0;
   while (a) {
     int rv = a->Prepare();
     if (rv)
       return rv;
 
+    float percent = float(++i) / float(mCount);
+    UpdateProgressUI(PROGRESS_PREPARE_SIZE * percent);
+
     a = a->mNext;
   }
 
-  UpdateProgressUI(1.0f);
-
   return OK;
 }
 
 int
 ActionList::Execute()
 {
-  int i = 0;
-  float divisor = mCount / 98.0f;
-
+  int currentProgress = 0, maxProgress = 0;
   Action *a = mFirst;
   while (a) {
-    UpdateProgressUI(1.0f + float(i++) / divisor);
+    maxProgress += a->mProgressCost;
+    a = a->mNext;
+  }
 
+  a = mFirst;
+  while (a) {
     int rv = a->Execute();
-    if (rv)
-    {
+    if (rv) {
       LOG(("### execution failed\n"));
       return rv;
     }
 
+    currentProgress += a->mProgressCost;
+    float percent = float(currentProgress) / float(maxProgress);
+    UpdateProgressUI(PROGRESS_PREPARE_SIZE +
+                     PROGRESS_EXECUTE_SIZE * percent);
+
     a = a->mNext;
   }
 
@@ -1432,8 +1754,15 @@ void
 ActionList::Finish(int status)
 {
   Action *a = mFirst;
+  int i = 0;
   while (a) {
     a->Finish(status);
+
+    float percent = float(++i) / float(mCount);
+    UpdateProgressUI(PROGRESS_PREPARE_SIZE +
+                     PROGRESS_EXECUTE_SIZE +
+                     PROGRESS_FINISH_SIZE * percent);
+
     a = a->mNext;
   }
 
@@ -1441,8 +1770,6 @@ ActionList::Finish(int status)
   if (status == OK)
     gSucceeded = TRUE;
 #endif
-
-  UpdateProgressUI(100.0f);
 }
 
 int DoUpdate()
@@ -1460,12 +1787,12 @@ int DoUpdate()
   if (rv)
     return rv;
 
-  AutoFD mfd = NS_topen(manifest, O_RDONLY | _O_BINARY);
-  if (mfd < 0)
+  AutoFile mfile = NS_tfopen(manifest, NS_T("rb"));
+  if (mfile == NULL)
     return READ_ERROR;
 
   struct stat ms;
-  rv = fstat(mfd, &ms);
+  rv = fstat(fileno(mfile), &ms);
   if (rv)
     return READ_ERROR;
 
@@ -1476,7 +1803,7 @@ int DoUpdate()
   int r = ms.st_size;
   char *rb = mbuf;
   while (r) {
-    int c = read(mfd, rb, mmin(SSIZE_MAX,r));
+    int c = fread(rb, 1, mmin(SSIZE_MAX,r), mfile);
     if (c < 0)
       return READ_ERROR;
 
